/********************************************************************************/
/*                                                                              */
/* irxradio.cmd - Internet Rexx Radio version 1.5.  by Wolfgang Reinken         */
/* last update: March 1st 2008                                                  */
/*                                                                              */
/* irxradio is a text based frontend for playing sound streams with mplayer.exe */
/*          as backend.                                                         */
/* irxradio's look and feel is similar to an old style radio with vacuum tubes. */
/*                                                                              */
/* - disclaimer -                                                               */
/* this program comes with absolutely no guarantees. any problems, side effects,*/
/* or whatever is your responsibility.                                          */
/*                                                                              */
/* Comments are welcome: wolfgang.reinken@t-online.de                           */
/*                                                                              */
/* 20141213 W.Eike Skalenbereich um zwei Zeilen erweitert                       */
/* 20141213 W.Eike PM123 als alternativen Player eingebaut siehe PLAYER         */
/* 20141213 W.Eike PM123 Song und Station von PM123 abrufen                     */
/* 20141213 W.Eike MPLAYER Songermittlung von IRXRADIO.WRK auf QUEUE umgeleitet */
/*                                                                              */
/*                                                                              */
/********************************************************************************/
"@mode 90,35"           /* text screen 90 columns, 35 rows */
call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

signal on HALT name Abort

/* ---------------------------------------------------------------------------- */
/* paint static parts of radio front                                            */
/* ---------------------------------------------------------------------------- */
call SysCurState "OFF"
call charout ,'1b'X'[0;37;40m'  /* background black, foreground white   */
call SysCls                     /* blank screen                         */

/*                                              */
/* paint the wooden chassis of the radio        */
/*                                              */
call charout ,'1b'X'[0;30;41m'  /* background dark red */
call SysCurPos 0, 0; call charout ,copies('',90)
call SysCurPos 1, 0; call charout ,copies('',90)
do i=2 to 32
  call SysCurPos i,  0; call charout ,''
  call SysCurPos i, 86; call charout ,''
end
call SysCurPos 33, 0; call charout ,copies('',90)
call SysCurPos 34, 0; call charout ,copies('',89)

/*                                              */
/* paint the bright wooden frames at the front  */
/*                                              */
call charout ,'1b'X'[1;30;43m'  /* background dark yellow, foreground gray */
call SysCurPos 2, 4; call charout ,copies('',82)
do i=3 to 31
  call SysCurPos i,  4; call charout ,''
  call SysCurPos i, 84; call charout ,''
end
call SysCurPos 15, 4; call charout ,copies('',82)
call SysCurPos 32, 4; call charout ,copies('',82)

/*                                              */
/* paint the loudspeaker area                   */
/*                                              */
call charout ,'1b'X'[1;33;43m'  /* background dark yellow, foreground yellow */
do i=3 to 14
  call SysCurPos i, 6; call charout ,copies('',39)
end
call SysCurPos 5, 31; call charout ,' IRXRADIO '


/*                                              */
/* paint the keys and key descriptions          */
/*                                              */
call SysCurPos 32, 28; call charout ,copies('',7)
call charout ,'1b'X'[1;33;40m'  /* background black, foreground yellow */
call SysCurPos 31, 28; call charout ,copies(' ',7)
call SysCurPos 30, 28; call charout ,' Qu   He   Di            In '

/* ---------------------------------------------------------------------------- */
/* initialize global variables                                                  */
/* ---------------------------------------------------------------------------- */
PID=0                   /* procedd ID of mplayer.exe                            */
ICYinfo=''              /* ICY info string (rad by mplayer.exe)                 */
ICYdelay=10
disp='D'                /* scale brightness dark (radio state switched off) */
stdisp=0                /* station display off */
key=" "
lastkey=" "
station=''
keydelay=0
p=10                    /* Scale position (overridden by irxradio.ini)          */
cachesize=0             /* standard cache size of mplayer                       */
proxy=""                /* proxy server                                         */
player="PM123.EXE"      /* welcher player? MPLAYER.EXE oder PM123.EXE           */
player="MPLAYER.EXE"    /* welcher player? MPLAYER.EXE oder PM123.EXE           */


/* ---------------------------------------------------------------------------- */
/* read initial files (irxradio.cfg and irxradio.ini)                           */
/* ---------------------------------------------------------------------------- */
call ReadConfig
call ReadIni

/* ---------------------------------------------------------------------------- */
/* paint variable parts of radio front                                          */
/* ---------------------------------------------------------------------------- */
call MakeScale          /* configured by irxradio.cfg */
call DispEye
call DispPointer

/* ---------------------------------------------------------------------------- */
/* load utilities                                                               */
/* ---------------------------------------------------------------------------- */
call rxfuncadd 'rxuinit','rxu','rxuinit'
signal on syntax name termrxu
call rxuinit
call RxFuncAdd 'RxExtra', 'RxExtras', 'RxExtra'
signal on syntax name termrxextras
call RxExtra 'Load'
signal off syntax
select
 when player = "MPLAYER.EXE" then,
  do
   /* klren ob mplayer installiert ist */
   '@detach mplayer.exe 1>nul 2>nul'
   if rc>0 then signal TermMplayer
  end
 when player = "PM123.EXE" then,
  do
   call RxFuncAdd 'PYRxInit','PYRxUtil','PYRxInit'
   call PYRxInit
   '@start /PM /MIN pm123.exe'
   call syssleep 1
   nH=PYRxOpen('\pipe\pm123','RW+')  /**/
   plin = '*HIDE'||'00'x
   call PYRxRW nH,'w',pLin     /**/
  end
 otherwise nop
end

/* ---------------------------------------------------------------------------- */
/* MAIN CYCLE (ends with key "Q" pressed (quit radio)                           */
/* ---------------------------------------------------------------------------- */
do while key<>"Q"
  /* -------------------------------------------------------------------------- */
  /* get info of pressed key                                                    */
  /* -------------------------------------------------------------------------- */
  KeyStr = RxKbdCharIn(nowait)  /* information of last pressed key      */
  Key=translate(word(KeyStr,words(KeyStr)-5))   /* ascii key code       */
  Keys=translate(word(KeyStr,words(KeyStr)-4))  /* key scan code        */
  Keysk=word(KeyStr,words(KeyStr)-2)            /* Byte contains shift key info */
  if substr(KeyStr,2,3)='   ' then Key=' '      /* space bar pressed    */
  SKpr=substr(x2b(right(Keysk,4,'0')),15,1)=1|substr(x2b(right(Keysk,4,'0')),16,1)=1      /* Shift Key gedrckt? */

  if keys="4D" then key="R"     /* right arrow pressed */
  if keys="4B" then key="L"     /* left arrow pressed  */
  if keys="01" then key="Q"     /* ESC   key   pressed */  /* add statement 2007-07-22 Rainer */

  if keydelay=0 then            /* release radio arrow keys */
  do
    call charout ,'1b'X'[1;33;40m'      /* background black, foreground yellow  */
    call SysCurPos 31, 53; call charout ," "  /* radio right arrow key      */
    call SysCurPos 31, 43; call charout ," "  /* radio left arrow key       */
  end

  /* -------------------------------------------------------------------------- */
  /* process info of pressed key                                                */
  /* -------------------------------------------------------------------------- */
  select
    when key="H" | keys="3B" then
      /* ---------------------------------------------------------------------- */
      /* display help                                                           */
      /* ---------------------------------------------------------------------- */
      do
        call charout ,'1b'X'[1;33;40m'  /* background black, foreground yellow  */
        call SysCurPos 31, 33; call charout ,"     "  /* radio help key         */
        call syssleep 1
        '@start view irxradio.hlp'
        call SysCurPos 31, 33; call charout ," "  /* radio help key         */
      end
    when keys="4D" then
      /* ---------------------------------------------------------------------- */
      /* scale pointer to the right                                             */
      /* ---------------------------------------------------------------------- */
      do
        if lastkey<>"R" then
        do
          call charout ,'1b'X'[1;33;40m' /* bg black, fg yellow */
          call SysCurPos 31, 53; call charout ,"     "
                        /* radio right arrow key pressed        */
        end
        keydelay=15     /* wait 15 cycles to release radio right arrow key */
        call ClearPointer
        if p<79 then p=p+1
        call DispPointer
      end
    when keys="4B" then
      /* ---------------------------------------------------------------------- */
      /* scale pointer to the left                                              */
      /* ---------------------------------------------------------------------- */
      do
        if lastkey<>"L" then
        do
          call charout ,'1b'X'[1;33;40m'        /* bg black, fg yellow */
          call SysCurPos 31, 43; call charout ,"     "
                                        /* radio left arrow key pressed */
        end
        keydelay=15     /* wait 15 cycles to release radio left arrow key */
        call ClearPointer
        if p>10  then p=p-1
        call DispPointer
      end
    when key="I" & disp="D" then
      /* ---------------------------------------------------------------------- */
      /* turn radio on (internet)                                               */
      /* ---------------------------------------------------------------------- */
      do
        call charout ,'1b'X'[1;33;40m'  /* bg black, fg yellow */
        call SysCurPos 31, 58; call charout ,"     "
                                        /* radio "internet" key pressed         */
        call Syssleep 2     /* wait 2 seconds for "warming up the vacuum tubes" */
        Disp='L'                        /* brightness of scale --> high         */
        call DispScale
        call DispPointer
        call syssleep 1 /* wait 1 second for "warming up the magic eye"         */
        call DispEye
      end
    when (key=" " | Keys="50" | key="0d"x ) & disp<>"D"  then
      /* ---------------------------------------------------------------------- */
      /* start playing                                                          */
      /* ---------------------------------------------------------------------- */
      do
        call charout ,'1b'X'[1;33;40m'  /* bg black, fg yellow */
        call SysCurPos 31, 48; call charout ,"     "
        do i=17 to 29
          if SysTextScreenRead(i,p-2,5)="" then
          do j=p-3 to 10 by -1
            c=SysTextScreenRead(i,j,1)
            if c2d(c)<160
              then station=c||station
              else j=1
          end
        end
        if length(station)>0 then
        do
          call SysCurPos 31, 53; call charout ," "
          call SysCurPos 31, 43; call charout ," "
          station=strip(station)
          call play station
        end
        else
        do
          call charout ,'1b'X'[1;33;40m'  /* bg black, fg yellow */
          call SysCurPos 31, 48; call charout ," "
        end
      end

    /* ------------------------------------------------------------------------ */
    /* switch station display on and off                                        */
    /* ------------------------------------------------------------------------ */
    when key="D" then call DispRemoveDisplay

    /* ------------------------------------------------------------------------ */
    /* turn radio off                                                           */
    /* ------------------------------------------------------------------------ */
    when key="Q" then call SwitchOff

    otherwise
  end
  lastkey=key
  Call RxNap 10                               /* cycle delay 0.01s              */
  if keydelay>0 then keydelay=keydelay-1      /* count down delay for releasing */
                                              /* radio arrow keys               */
end                                     /* END MAIN CYCLE                       */

/* ---------------------------------------------------------------------------- */
/* clean up the program and end                                                 */
/* ---------------------------------------------------------------------------- */
call WriteIni                   /* save program state */

call charout ,'1b'X'[0;37;40m'  /* background black   */
call SysCls

exit

/********************************************************************************/
/* Create and display the radio scale                                           */
/* - process the values of irxradio.cfg                                         */
/* - find correct positions for station name (ShortName) and station bar   */
/********************************************************************************/
MakeScale:
  i=18                          /* first scale line     */
  j=10                          /* left scale position  */
  imax=27                       /* last scale line      */
  /* background black, foreground yellow */
  if disp='D' then call charout ,'1b'X'[0;33;40m' /* bg black, fg dark yellow   */
  if disp='L' then call charout ,'1b'X'[1;33;40m' /* bg black, fg yellow        */
  if disp='R' then call charout ,'1b'X'[1;33;40m' /* bg black, fg yellow        */

  do k=1 to ShortName.0         /* cycle per entry in irxradio.cfg              */
    LenShortName=length(ShortName.k)+8  /* 8 = 5 bytes station bar + 1 space left + 2 spces right */
    if j+LenShortName>80 then   /* no space on actual scale line left   */
    do
      i=i+1                     /* next scale line                      */
      select
        when i>imax then leave          /* config file is too long      */
        when i-18<5 then j=10-18+i      /* initial left position        */
        when i-18>4 then j=10-23+i      /* in scal line                 */
        otherwise
      end
    end
    do l=18 to i-1              /* check if position of station bar is already used above */
      if pos(" ",SysTextScreenRead(l,j+length(ShortName.k),6))=1 then
      do                        /* if used then ... */
        j=j+1                   /* ... increment left position  */
        l=17                    /* ... restart comparisn        */
      end
    end
    call SysCurPos i, j; say ShortName.k" "  /* display station name and station bar */
    j=j+LenShortName
  end
return

/********************************************************************************/
/* Display radio scale created by MakeScal in different brightness              */
/********************************************************************************/
DispScale:
  if disp='D' then call charout ,'1b'X'[0;33;40m'
  if disp='L' then call charout ,'1b'X'[1;33;40m'
  if disp='R' then call charout ,'1b'X'[1;33;40m'
  do i=17 to 29
    c=SysTextScreenRead(i,10,74)
    call SysCurPos i, 10; call charout ,c
  end
return

/********************************************************************************/
/* Display radio scale pointer at position p                                    */
/********************************************************************************/
DispPointer:
  if disp='D' then call charout ,'1b'X'[0;31;40m'      /* bg black, fg dark red */
  if disp='L' then call charout ,'1b'X'[1;31;40m'      /* bg black, fg red      */
  if disp='R' then call charout ,'1b'X'[1;31;40m'      /* bg black, fg red      */
  do i=17 to 27
    c=SysTextScreenRead(i,p,1)                  /* read screen character        */
    call SysCurPos i, p; call charout ,c        /* display in (dark) red        */
  end
return

/********************************************************************************/
/* Clear radio scale pointer at position p                                      */
/********************************************************************************/
ClearPointer:
  if disp='D' then call charout ,'1b'X'[0;33;40m'   /* bg black, fg dark yellow */
  if disp='L' then call charout ,'1b'X'[1;33;40m'   /* bg black, fg yellow      */
  if disp='R' then call charout ,'1b'X'[1;33;40m'   /* bg black, fg yellow      */
  do i=17 to 27
    c=SysTextScreenRead(i,p,1)                  /* read screen character        */
    call SysCurPos i, p; call charout ,c        /* display in (dark) yellow     */
  end
return

/********************************************************************************/
/* Display magic eye in one of 3 states:                                        */
/* - disp='D' (radio switched off): colour gray                                 */
/* - disp='L' (radio switched on):  colour dark green                           */
/* - disp='R' (radio playing):      colour light green                          */
/********************************************************************************/
DispEye:
  if disp='D' then call charout ,'1b'X'[1;30;43m'  /* bg dark yellow, fg gray       */
  if disp='L' then call charout ,'1b'X'[0;32;43m'  /* bg dark yellow, fg dark green */
  if disp='R' then call charout ,'1b'X'[0;32;43m'  /* bg dark yellow, fg dark green */
  call SysCurPos 11, 10; call charout ,'  '
  call SysCurPos 12, 10; call charout ,'  '
  call charout ,'1b'X'[1;32;43m'                  /* bg dark yellow, fg light green */
  if disp='L' then
  do
    /*call SysCurPos 11, 17; call charout ,''*/
    /*call SysCurPos 12, 17; call charout ,''*/
    call SysCurPos 11, 11; call charout ,''
    call SysCurPos 11, 23; call charout ,''
    call SysCurPos 12, 11; call charout ,''
    call SysCurPos 12, 23; call charout ,''
  end
  if disp='R' then
  do
    /*call SysCurPos 11, 12; call charout ,''*/
    /*call SysCurPos 12, 12; call charout ,''*/
    call SysCurPos 11, 11; call charout ,''
    call SysCurPos 11, 18; call charout ,''
    call SysCurPos 12, 11; call charout ,''
    call SysCurPos 12, 18; call charout ,''
  end
return

/********************************************************************************/
/* Process irxradio.cfg                                                         */
/* Syntax of irxradio.cfg:                                                      */
/* - line starting with #<keyword>              configuration line              */
/* - line starting at line position 1 (w/o #):  station name (ShortName)        */
/* - following line (first character = " "):    station url                     */
/*   - if following line is missing, only station name is displayed at scale    */
/********************************************************************************/
ReadConfig:
  i=0
  do while lines("irxradio.cfg")>0
    c=linein("irxradio.cfg")
    select
      when translate(word(c,1))="#PROXY"        /* line with proxy server url   */
        then proxy="http_proxy://"word(c,2)"/"
      when translate(word(c,1))="#CACHE" then   /* line with cache parameter    */
        if word(c,2) >= 32 & word(c,2) <= 3000
          then cachesize=word(c,2)
          else cachesize=0
      when left(c,1)="#"                        /* line with invalid parameter  */
        then nil
      when left(c,1)<>" " then                  /* line with station name       */
        do
          i=i+1
          ShortName.i=strip(c)
        end
      otherwise                                 /* line with station url        */
        url.i=strip(c)
    end  /* select */
  end
  ShortName.0=i
  RC = STREAM("irxradio.cfg",'C','CLOSE')
  /* call linein "irxradio.cfg"  close */
return

/********************************************************************************/
/* Read inifile irxradio.ini and get initial values                             */
/* - scalepos : position of scale pointer in previos run of the program         */
/********************************************************************************/
ReadIni:
  i=0
  do while lines("irxradio.ini")>0
    c=linein("irxradio.ini")
    if word(c,1)="scalepos " then p=word(c,2)
    i=i+1
  end
  RC = STREAM("irxradio.ini",'C','CLOSE')
  /* call linein "irxradio.ini"     close */
return

/********************************************************************************/
/* Write values of variables to inifile irxradio.ini                            */
/* - scalepos : actual position of scale pointer                                */
/********************************************************************************/
WriteIni:
  '@DEL irxradio.ini 1>nul 2>nul '
  call lineout "irxradio.ini","scalepos "p
  RC = STREAM("irxradio.ini",'C','CLOSE')
  /* call linein "irxradio.ini"     close */
return

/********************************************************************************/
/* Display station display on the loudspeaker area                              */
/* - first line:  station name (centerd)                                        */
/* - second line: ICY-info "Titlename" if available (centerd, cut if longer 40) */
/********************************************************************************/
DispStDisplay:
  call charout ,'1b'X'[0;30;43m'                /* bg dark yellow, fg black     */
  call SysCurPos  9, 30; call charout ,'  '
  call SysCurPos 10, 30; call charout ,'  '
  call SysCurPos 11, 30; call charout ,'  '
  call SysCurPos 12, 30; call charout ,'  '
  if disp="R" then
  do
    call charout ,'1b'X'[1;32;40m'              /* bg black, fg light green     */
    lpos=(40-length(station))%2
    call SysCurPos 10, 31+lpos; call charout ,station
    lpos=(40-length(ICYinfo))%2
    if lpos<0 then lpos=0
    ICYlen=length(ICYinfo)
    ICYpointer=1
    ICYdelay=10
    call SysCurPos 11, 31+lpos
    if lpos=0
      then call charout ,left(ICYinfo,40)
      else call charout ,ICYinfo
  end
return

/********************************************************************************/
/* Remove station display from the loudspeaker area                             */
/********************************************************************************/
RemoveStDisplay:
  call charout ,'1b'X'[1;33;43m'                /* bg dark yellow, fg yellow */
  do i=9 to 12
    call SysCurPos i, 30; call charout ,copies('',22)
  end
return

/********************************************************************************/
/* Play a radio station (url) with mplayer.exe                                  */
/********************************************************************************/
Play:
  ICYinfoOld=ICYinfo
  do i=1 to Shortname.0
    /* ShortName was read from irxradio.cfg                             */
    /* station is text left of station bar of actual pointer position   */
    if Shortname.i=station then
    do
      if word(url.i,1)="-playlist"
        then do; playlst="-playlist"; n=2; end
        else do; playlst=""; n=1; end
      urli=word(url.i,n)
      caci=word(url.i,n+1)
    end
  end
  select
   when player = "MPLAYER.EXE" then,
    do
     rc=stream('irxradio.wrk','c','open')  /* test if workfile is writable */
     if rc<>'READY:'
       then call SysSleep 2                /* prevents a NOT READY error of the following detach command */
       else rc=stream('irxradio.wrk','c','close')

     /* -------------------------------------------------------------------------- */
     /* start mplayer.exe in background, redirect output to QUEUE IRXQUEUE         */
     /* -------------------------------------------------------------------------- */
     if cachesize>0
       then cacheparm="-cache "cachesize
       else cacheparm=""
     if caci >= 32 & caci <= 3000
       then cacheparm="-cache "caci
     IRXQUEUE = 'IRXRADIO'
     rc = rxqueue('DELETE', IRXQUEUE)
     rc = rxqueue('CREATE', IRXQUEUE)
     rc = rxqueue('SET', IRXQUEUE)
     call charout ,'1b'X'[0;30;41m'  /* background dark red */
     call SysCurPos 0, 0; call charout ,copies('',90)
     call SysCurPos 0, 0
     '@detach mplayer.exe  -ao kai:dart -quiet -playlist' urli  '| rxqueue' irxqueue
     rc = syssleep(1)
     call charout ,'1b'X'[0;30;41m'  /* background dark red */
     call SysCurPos 0, 0; call charout ,copies('',90)
    end
   when player = "PM123.EXE" then,
    do
     PID=GetPID()          /* process id of player check if still alive       */
     if PID=0 then,
      do
       call PYRxRW nH,'w'          /*close old pipe*/
       '@start /PM /MIN pm123.exe'
       call syssleep 1
       nH=PYRxOpen('\pipe\pm123','RW+')  /**/
       plin = '*HIDE'||'00'x
       call PYRxRW nH,'w',pLin     /**/
      end
     plin = '*LOAD' proxy||urli||'00'x
     call PYRxRW nH,'w',pLin     /**/
    end
   otherwise nop
  end
  PID=GetPID()          /* process id of player for later kill operation   */
  PIDa=PID
  if PID>0 then
  do
    disp="R"
  end
  call DispEye
  if stdisp=1 then call DispStDisplay
  cdelay=1                         /* delay to reduce system load while playing */

  /* -------------------------------------------------------------------------- */
  /* PLAY CYCLE                                                                 */
  /* cycle until ...                                                            */
  /* - operator intervention (press a key) or                                   */
  /* - end of transmission (PID=0)                                              */
  /* -------------------------------------------------------------------------- */
  ICYincr = 1
  do until (c=' ' | s='4B'| s='4D' | s='50' | c='Q' | c='0d'x | disp='L')
    KeyStr = RxKbdCharIn(nowait)                /* input of one character w/o echo; with analysing scan code */
    c=translate(word(KeyStr,words(KeyStr)-5))   /* ascii key code       */
    s=translate(word(KeyStr,words(KeyStr)-4))   /* key scan code        */
    sk=word(KeyStr,words(KeyStr)-2)             /* Byte contains shift key info */
    if substr(KeyStr,2,3)='   ' then c=' '      /* space bar pressed    */
    SKpr=substr(x2b(right(sk,4,'0')),15,1)=1|substr(x2b(right(sk,4,'0')),16,1)=1      /* Shift Key gedrckt? */
    if s="01" then c=" "        /* ESC   key   pressed */  /* add statement 2007-07-22 Rainer */

    /* Switch display on / off */
    if c='D' then call DispRemoveDisplay

    /* display help                                                             */
    if c="H" | s="3B" then
    do
      call charout ,'1b'X'[1;33;40m'    /* background black, foreground yellow  */
      call SysCurPos 31, 33; call charout ,"     "  /* radio help key           */
      call syssleep 1
      '@start view irxradio.hlp'
      call SysCurPos 31, 33; call charout ," "  /* radio help key           */
    end

    /* display long ICYinfo     */
    if stdisp=1 & ICYlen>40 then
    do
      select
        when ICYdelay>0 then ICYdelay=ICYdelay-1
        when ICYdelay=0 & ICYlen-ICYpointer>=40 then
          do
            ICYpointer=ICYpointer+ICYincr
            if ICYpointer < 1 then,
             do
              ICYdelay=10
              ICYpointer=1
              ICYincr=1
             end
            call SysCurPos 11, 31
            call charout ,substr(ICYinfo,ICYpointer,40)
          end
        when ICYdelay=0 & ICYlen-ICYpointer=39 then ICYdelay=ICYdelay-1
        when ICYdelay<0 & ICYdelay>-10 then ICYdelay=ICYdelay-1
        when ICYdelay<(-9) then
          do
            ICYdelay=0
            ICYincr=-1
            ICYpointer=ICYpointer+ICYincr
            call SysCurPos 11, 31
            call charout ,substr(ICYinfo,ICYpointer,40)
          end
        otherwise
      end
    end

    /* Test if mplayer.exe is still running */
    PID=GetPID()
    if PID<>PIDa then
    do
      if PID=0
        then disp='L'
        else disp='R'
      call DispEye
      if stdisp=1 then call DispStDisplay
    end

    /* refresh ICYinfo and stationdisplay every 4s (20*200ms) */
    if cdelay<1 then
    do
     select
      when player = "MPLAYER.EXE" then,
       do queued()
        parse pull izeile
        if pos('StreamTitle=',izeile) = 0 then iterate
        parse var izeile . "StreamTitle='" ICYinfo "';" .
       end
      when player = "PM123.EXE" then,
       do
        pLin = '*INFO META'||'00'x
        call PYRxRW nH,'w',pLin     /**/
        rd_data = PYRxRW(nH,'r')
        rd_data = translate(rd_data,'  ','0a'x||'00'x)
        parse var rd_data rd_data 'GENRE=' rest
        parse var rd_data rd_data 'COMMENT=' station
        parse var rd_data  .      'TITLE=' ICYinfo
        ICYinfo = strip(ICYinfo)
        if ICYinfo = '' then ICYinfo = station
        if length(station) > 40 then station = left(station,40)
        station = center(strip(station),40)
       end
      otherwise nop
     end

      if ICYinfoOld<>ICYinfo & stdisp=1 then
      do
        call DispStDisplay
        ICYinfoOld=ICYinfo
      end
      cdelay=20
    end
    else cdelay=cdelay-1

    Call RxNap 200
  end                   /* END PLAY CYCLE */

  /* -------------------------------------------------------------------------- */
  /* clean up playing routine                                                   */
  /* -------------------------------------------------------------------------- */
  station=''
  disp="L"
  ICYinfo=''
  select
   when player = 'PM123.EXE' then,
    do
     rd_data = PYRxRW(nH,'r')
     pLin = '*REMOVE'||'00'x
     call PYRxRW nH,'w',pLin     /**/
     rd_data = PYRxRW(nH,'r')
    end
   when player = 'MPLAYER.EXE' then,
    do
     killrc = RxKillProcess(PID)
    end
   otherwise nop
  end
  call charout ,'1b'X'[1;33;40m'
  call SysCurPos 31, 48; call charout ," "
  call DispEye
  if stdisp=1 then call DispStDisplay
  if c='Q' then
  do
    call SwitchOff
    key="Q"
  end
return

/********************************************************************************/
/* Get the process id of player                                                 */
/********************************************************************************/
GetPID:
      GetPIDNr=0
      pl = length(player)
      call RxQProcStatus proc.
      do przi=1 to proc.0p.0
        if right(proc.0p.przi.6,pl)=PLAYER then,
         do
          GetPIDNr=x2d(proc.0p.przi.1)
          leave
         end
      end
return GetPIDNr

/********************************************************************************/
/* Display station display if switched off                                      */
/* Remove station display if switched on                                        */
/********************************************************************************/
DispRemoveDisplay:
      do
        select
          when stdisp=0 then
            do
              call charout ,'1b'X'[1;33;40m'
              call SysCurPos 31, 38; call charout ,"     "
              stdisp=1
              call DispStDisplay
            end
          when stdisp=1 then
            do
              call charout ,'1b'X'[1;33;40m'
              call SysCurPos 31, 38; call charout ," "
              stdisp=0
              call RemoveStDisplay
            end
          otherwise
        end
      end
return

/********************************************************************************/
/* Switch off radio:                                                            */
/* - release all radio keys                                                     */
/* - switch scale and magic eye to dark brightness                              */
/* - kill mplayer.exe (stop playing)                                            */
/* - switch station display off                                                 */
/********************************************************************************/
SwitchOff:
      do
        call charout ,'1b'X'[1;33;40m'
        call SysCurPos 31, 28; call charout ,"     "
        call SysCurPos 31, 33; call charout ,copies(' ',6)
        Disp='D'
        call DispScale
        call DispPointer
        call DispEye
        select
         when player = 'PM123.EXE' then,
          do
           pLin = '*REMOVE'||'00'x
           call PYRxRW nH,'w',pLin     /**/
           call PYRxRW nH,'w'          /* pipe close */
          end
         otherwise nop
        end
        killrc = RxKillProcess(PID)
        call RemoveStDisplay
        call Syssleep 1
        call charout ,'1b'X'[1;33;40m'
        call SysCurPos 31, 28; call charout ," "
        call Syssleep 2
      end
return


/********************************************************************************/
/* End program if <CTRL>-C was pressed                                          */
/********************************************************************************/
Abort:
  call SwitchOff
  /* -------------------------------------------------------------------------- */
  /* clean up the program and end                                               */
  /* -------------------------------------------------------------------------- */
  call WriteIni                         /* save program state */

  call charout ,'1b'X'[0;37;40m'        /* background black   */
  call SysCls
  exit

TermRXU:
  call charout ,'1b'X'[0;37;40m'        /* background black   */
  call SysCurPos 28, 10; call charout ,"Library RXU is missing"
  call SysCurPos 29, 10; call charout ,"Please obtain from: http://hobbes.nmsu.edu/pub/os2/dev/rexx/rxu1a.zip"
  call SysCurPos 33, 5
  exit

TermRxExtras:
  call charout ,'1b'X'[0;37;40m'        /* background black   */
  call SysCurPos 28, 10; call charout ,"Library RXEXTRAS is missing"
  call SysCurPos 29, 10; call charout ,"Please obtain from: http://hobbes.nmsu.edu/pub/os2/dev/rexx/rxx1g.zip"
  call SysCurPos 33, 5
  exit

TermMplayer:
  call charout ,'1b'X'[0;37;40m'        /* background black   */
  call SysCurPos 28, 10; call charout ,"Program MPLAYER is missing"
  call SysCurPos 29, 10; call charout ,"Please obtain the most recent from: http://hobbes.nmsu.edu/cgi-bin/h-browse?dir=/pub/os2/apps/mmedia/video/players"
  call SysCurPos 33, 5
  exit
